unit QuickenPanel;  { TQuickenPanel component. }
{ Created 2003-07-18 13:47:34 by Icebird@cnblogs }
{ Eagle Software CDK, Version 7.01 Rev. B (Professional Version) }

interface

uses
  Windows, 
  SysUtils, 
  Messages, 
  Classes, 
  Graphics, 
  Controls, 
  Forms, 
  Dialogs, 
  Menus, 
  StdCtrls, 
  ExtCtrls;

const
  FRound = 15; // Radius of round corner

type
  TPictureAlign = (paLeft, paRight);  { Picture Align }

  TQuickenPanel = class(TCustomControl)
  private
    FHasRound: boolean;
    FHeaderColor: TColor;
    FHeaderFontColor: TColor;
    FHeadPopupMenu: TPopupMenu;
    FMainColor: TColor;
    FOnImageClick: TNotifyEvent;
    FPicture: TPicture;
    FPictureAlign: TPictureAlign;
    FPopUpBtnColor: TColor;
    FPopUpBtnFontColor: TColor;
    FPopUpBtnText: string;
    FTitle: string;
    procedure DrawPanel;
    function GetPopUpBtnWidth: Integer;
    procedure PictureChanged(Sender: TObject);
    procedure SetRgn(R: Integer);
    procedure WMLButtonDown(var Msg: TWMLButtonDown); message WM_LBUTTONDOWN;
  protected
    procedure AdjustClientRect(var Rect: TRect); override;
    procedure CreateWnd; override;
    function GetHeaderHeight: Integer; virtual;
    procedure ImageClick; dynamic;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Paint; override;
    procedure SetHasRound(newValue: boolean); virtual;
    procedure SetHeaderColor(newValue: TColor); virtual;
    procedure SetHeaderFontColor(newValue: TColor); virtual;
    procedure SetHeadPopupMenu(newValue: TPopupMenu); virtual;
    procedure SetMainColor(newValue: TColor); virtual;
    procedure SetPicture(newValue: TPicture); virtual;
    procedure SetPictureAlign(newValue: TPictureAlign); virtual;
    procedure SetPopUpBtnColor(newValue: TColor); virtual;
    procedure SetPopUpBtnFontColor(newValue: TColor); virtual;
    procedure SetPopUpBtnText(newValue: string); virtual;
    procedure SetTitle(newValue: string); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    property Enabled;
    property Font;
    property HasRound: boolean read FHasRound write SetHasRound;  { Published }
    property HeaderColor: TColor read FHeaderColor write SetHeaderColor;  { Published }
    property HeaderFontColor: TColor read FHeaderFontColor write SetHeaderFontColor;  { Published }
    property HeaderHeight: Integer read GetHeaderHeight;  { Published }
    property HeadPopupMenu: TPopupMenu read FHeadPopupMenu write SetHeadPopupMenu;  { Published }
    property MainColor: TColor read FMainColor write SetMainColor;  { Published }
    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnImageClick: TNotifyEvent read FOnImageClick write FOnImageClick;  { Published }
    property OnKeyDown;
    property OnKeyPress;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property Picture: TPicture read FPicture write SetPicture;  { Published }
    property PictureAlign: TPictureAlign read FPictureAlign write SetPictureAlign;  { Published }
    property PopUpBtnColor: TColor read FPopUpBtnColor write SetPopUpBtnColor;  { Published }
    property PopUpBtnFontColor: TColor read FPopUpBtnFontColor write SetPopUpBtnFontColor;  { Published }
    property PopUpBtnText: string read FPopUpBtnText write SetPopUpBtnText;  { Published }
    property PopupMenu;
    property ShowHint;
    property Title: string read FTitle write SetTitle;  { Published }
    property Visible;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QuickenUI', [TQuickenPanel]);
end;    { Register }

procedure TQuickenPanel.DrawPanel;
var
  hr2: THandle;
  p: array[0..2] of TPoint;
  TriangleLeft, BmpWidth, SplitPos: Integer;
begin
  if FHasRound then
    SetRgn(FRound)
  else
    SetRgn(0);
  Color := FMainColor;
  Canvas.Font := Self.Font;
  Canvas.Brush.Color := Color;
  Canvas.FillRect(GetClientRect);
  Canvas.Brush.Color := FHeaderColor;
  Canvas.FillRect(Rect(0, 0, Width, GetHeaderHeight));
  if Trim(FTitle) <> '' then
  begin
    Canvas.Font.Color := FHeaderFontColor;
    Canvas.Font.Style := Canvas.Font.Style + [fsBold];
    if (FPicture.Graphic <> nil) and (FPictureAlign = paLeft) then
      BmpWidth := FPicture.Graphic.Width + 1
    else
      BmpWidth := 0;
    Canvas.TextOut(6 + BmpWidth, 4, FTitle);
  end;
  if FHeadPopupMenu <> nil then
    SplitPos := Width - GetPopUpBtnWidth
  else
    SplitPos := Width;
  if FPicture.Graphic <> nil then
  begin
    FPicture.Graphic.Transparent := True;
    if FPictureAlign = paRight then
      Canvas.Draw(SplitPos - FPicture.Graphic.Width - 2, 2, FPicture.Graphic)
    else
      Canvas.Draw(6, 2, FPicture.Graphic);
  end;
  if FHeadPopupMenu <> nil then
    if FHeadPopupMenu.Items.Count > 0 then
    begin
      Canvas.Brush.Color := FPopUpBtnColor;
      Canvas.FillRect(Rect(SplitPos, 0, Width, GetHeaderHeight));
      if FPopUpBtnColor <> FMainColor then
      begin
        // Draw light border
        Canvas.Pen.Color := FPopUpBtnColor + $292529;
        Canvas.MoveTo(SplitPos, 0);
        Canvas.LineTo(SplitPos, GetHeaderHeight);
      end;
      if Trim(FPopUpBtnText) <> '' then
      begin
        Canvas.Font.Style := Canvas.Font.Style - [fsBold];
        Canvas.Font.Color := FPopUpBtnFontColor;
        Canvas.TextOut(SplitPos + 6, 4, FPopUpBtnText);
      end;
      // Draw triangle
      TriangleLeft := Width - 18;
      p[0] := Point(TriangleLeft, GetHeaderHeight div 2);
      p[1] := Point(TriangleLeft + 8, GetHeaderHeight div 2);
      p[2] := Point(TriangleLeft + 4, GetHeaderHeight div 2 + 4);
      Canvas.Brush.Color := clBlack;
      hr2 := CreatePolygonRgn(p, 3, ALTERNATE);
      FillRgn(Canvas.Handle, hr2, Canvas.Brush.Handle);
      DeleteObject(hr2);
    end;
  if FPopUpBtnColor = FMainColor then
  begin
    // Draw boundary of title and client area
    Canvas.Pen.Color := clBlack;
    Canvas.MoveTo(2, GetHeaderHeight - 1);
    Canvas.LineTo(Width - 4, GetHeaderHeight - 1);
  end;
end;

function TQuickenPanel.GetPopUpBtnWidth: Integer;
begin
  Result := Canvas.TextWidth(FPopUpBtnText) + 28;
end;

procedure TQuickenPanel.PictureChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TQuickenPanel.SetRgn(R: Integer);
var
  hr3, hr1, hr2: THandle;
begin
  if R = 0 then
    hr1 := CreateRectRgn(0, 0, Width, GetHeaderHeight)
  else
    hr1 := CreateRoundRectRgn(0, 0, Width, GetHeaderHeight, R, R);
  hr2 := CreateRectRgn(0, GetHeaderHeight div 2, Width - 1, Height);
  hr3 := hr1;
  CombineRgn(hr3, hr1, hr2, RGN_OR);
  SetWindowRgn(Handle, hr3, True);
  DeleteObject(hr1);
  DeleteObject(hr2);
  DeleteObject(hr3);
end;

procedure TQuickenPanel.WMLButtonDown(var Msg: TWMLButtonDown);  { private }
var
  SplitPos, ImagePos: Integer;
  p: TPoint;
begin
  inherited;
  with Msg do
  begin
    p.X := XPos;
    p.Y := YPos;
    SplitPos := Width;
    if Keys = MK_LBUTTON then
    begin
      if FHeadPopupMenu <> nil then
        if FHeadPopupMenu.Items.Count > 0 then
        begin
          SplitPos := Width - GetPopUpBtnWidth;
          if (p.X <= Width - 2) and (p.X >= SplitPos + 2) and (p.Y <= GetHeaderHeight - 2) and (p.Y >= 2) then
          begin
            p.X := Width - 1;
            p.Y := GetHeaderHeight;
            //MapWindowPoints(Handle, 0, p, 2);
            P := Self.ClientToScreen(P);
            FHeadPopupMenu.Alignment := Menus.paRight;
            FHeadPopupMenu.Popup(p.X, p.Y);
          end;
        end;
      if FPicture.Graphic <> nil then
      begin
        if FPictureAlign = paLeft then
          ImagePos := 6
        else
          ImagePos := SplitPos - FPicture.Graphic.Width - 2;
        if (p.X <= ImagePos + FPicture.Graphic.Width) and (p.X >= ImagePos) and (p.Y >= 2) and (p.Y <= FPicture.Graphic.Height + 2) then
          ImageClick;
      end;
    end;
  end;
end;  { WmLButtonDown }

procedure TQuickenPanel.AdjustClientRect(var Rect: TRect);
begin
  inherited AdjustClientRect(Rect);
  Inc(Rect.Top, GetHeaderHeight);
  Dec(Rect.Right);
end;  { AdjustClientRect }

procedure TQuickenPanel.CreateWnd; 
begin
  inherited CreateWnd;
  DrawPanel;
end;  { CreateWnd }

function TQuickenPanel.GetHeaderHeight: Integer;
begin
  Result := Canvas.TextHeight('0') + 8;
end;  { GetHeaderHeight }

procedure TQuickenPanel.ImageClick;  { protected }
begin
  if Assigned(FOnImageClick) then
    FOnImageClick(Self);
end;  { ImageClick }

procedure TQuickenPanel.Notification(AComponent: TComponent; Operation: TOperation); 
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FHeadPopupMenu) then
  begin
    FHeadPopupMenu := nil;
    Invalidate;
  end;
end;  { Notification }

procedure TQuickenPanel.Paint; 
begin
  inherited Paint;
  DrawPanel;
end;  { Paint }

procedure TQuickenPanel.SetHasRound(newValue: boolean);
{ Sets data member FHasRound to newValue. }
begin
  if FHasRound <> newValue then
  begin
    FHasRound := newValue;
    Invalidate;
  end;
end;  { SetHasRound }

procedure TQuickenPanel.SetHeaderColor(newValue: TColor);
{ Sets data member FHeaderColor to newValue. }
begin
  if FHeaderColor <> newValue then
  begin
    FHeaderColor := newValue;
    Invalidate;
  end;
end;  { SetHeaderColor }

procedure TQuickenPanel.SetHeaderFontColor(newValue: TColor);
{ Sets data member FHeaderFontColor to newValue. }
begin
  if FHeaderFontColor <> newValue then
  begin
    FHeaderFontColor := newValue;
    Invalidate;
  end;
end;  { SetHeaderFontColor }

procedure TQuickenPanel.SetHeadPopupMenu(newValue: TPopupMenu);
{ Sets data member FHeadPopupMenu to newValue. }
begin
  if FHeadPopupMenu <> newValue then
  begin
    FHeadPopupMenu := newValue;
    Invalidate;
  end;
end;  { SetHeadPopupMenu }

procedure TQuickenPanel.SetMainColor(newValue: TColor);
{ Sets data member FMainColor to newValue. }
begin
  if FMainColor <> newValue then
  begin
    FMainColor := newValue;
    Invalidate;
  end;
end;  { SetMainColor }

procedure TQuickenPanel.SetPicture(newValue: TPicture);
{ Sets data member FPicture to newValue. }
begin
  FPicture.Assign(newValue);
end;  { SetPicture }

procedure TQuickenPanel.SetPictureAlign(newValue: TPictureAlign);
{ Sets data member FPictureAlign to newValue. }
begin
  if FPictureAlign <> newValue then
  begin
    FPictureAlign := newValue;
    Invalidate;
  end;
end;  { SetPictureAlign }

procedure TQuickenPanel.SetPopUpBtnColor(newValue: TColor);
{ Sets data member FPopUpBtnColor to newValue. }
begin
  if FPopUpBtnColor <> newValue then
  begin
    FPopUpBtnColor := newValue;
    Invalidate;
  end;
end;  { SetPopUpBtnColor }

procedure TQuickenPanel.SetPopUpBtnFontColor(newValue: TColor);
{ Sets data member FPopUpBtnFontColor to newValue. }
begin
  if FPopUpBtnFontColor <> newValue then
  begin
    FPopUpBtnFontColor := newValue;
    Invalidate;
  end;
end;  { SetPopUpBtnFontColor }

procedure TQuickenPanel.SetPopUpBtnText(newValue: string);
{ Sets data member FPopUpBtnText to newValue. }
begin
  if FPopUpBtnText <> newValue then
  begin
    FPopUpBtnText := newValue;
    Invalidate;
  end;
end;  { SetPopUpBtnText }

procedure TQuickenPanel.SetTitle(newValue: string);
{ Sets data member FTitle to newValue. }
begin
  if FTitle <> newValue then
  begin
    FTitle := newValue;
    Invalidate;
  end;
end;  { SetTitle }

constructor TQuickenPanel.Create(AOwner: TComponent);
{ Creates an object of type TQuickenPanel, and initializes properties. }
begin
  inherited Create(AOwner);
  { Create property fields (that are objects): }
  FPicture := TPicture.Create;
  ControlStyle := [csAcceptsControls, csCaptureMouse, csClickEvents, csDoubleClicks, csOpaque];
  Width := 200;
  Height := 200;
  FHasRound := True;
  FHeaderColor := $5A7D5A;
  FHeaderFontColor := clWhite;
  FPopUpBtnColor := $94B294;
  FPopUpBtnFontColor := clWindowText;
  FPopUpBtnText := 'Action';
  FTitle := ClassName;
  FMainColor := clWhite;
  FPictureAlign := paRight;
  FPicture.OnChange := PictureChanged;
end;  { Create }

destructor TQuickenPanel.Destroy;
begin
  { Free member variables: }
  FPicture.Free;
  inherited Destroy;
end;  { Destroy }

end.

